import socket
import time

class KD3305P:
	def __init__(self):
		try:
			#KORAD KD3305P UPD/IP configuration
			self.serverAddressPort = ("192.168.1.198", 18190)

			# Create a UDP socket at client side
			self.udpSocket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
			self.udpSocket.bind(('0.0.0.0', 18190))
		except:
			print("Failed UDP Connection")

	# Get Device Info
	def GetDeviceInfo(self):
		data = ":SYST:DEVINFO?\r\n"
		self.udpSocket.sendto(str.encode(data), self.serverAddressPort)
		dhcp = self.udpSocket.recvfrom(128)
		ip, addr = self.udpSocket.recvfrom(128)
		netmask, addr = self.udpSocket.recvfrom(128)
		gateway, addr = self.udpSocket.recvfrom(128)
		mac, addr = self.udpSocket.recvfrom(128)
		port, addr = self.udpSocket.recvfrom(128)
		baudrate, addr = self.udpSocket.recvfrom(128)
		print(dhcp)
		print(ip)
		print(netmask)
		print(gateway)
		print(mac)
		print(port)
		print(baudrate)

	# Set voltage of channel
	def SetVoltage(self, channel, voltage):
		if channel != 1 and channel != 2:
			# Invalid channel
			print("Invalid channel")
			return
		data = "VSET%d:%2.2f\r\n" % (channel, voltage)
		self.udpSocket.sendto(str.encode(data), self.serverAddressPort)

	# Get programmed voltage of channel
	def GetVoltage(self, channel):
		if channel != 1 and channel != 2:
			# Invalid channel
			print("Invalid channel")
			return
		data = "VSET%d?\r\n" % (channel)
		self.udpSocket.sendto(str.encode(data), self.serverAddressPort)
		response, addr = self.udpSocket.recvfrom(128)
		return float(response)
	
	# Set current of channel
	def SetCurrent(self, channel, voltage):
		if channel != 1 and channel != 2:
			# Invalid channel
			print("Invalid channel")
			return
		data = "ISET%d:%1.3f\r\n" % (channel, voltage)
		self.udpSocket.sendto(str.encode(data), self.serverAddressPort)

	# Get programmed current of channel
	def GetCurrent(self, channel):
		if channel != 1 and channel != 2:
			# Invalid channel
			print("Invalid channel")
			return
		data = "ISET%d?\r\n" % (channel)
		self.udpSocket.sendto(str.encode(data), self.serverAddressPort)
		response, addr = self.udpSocket.recvfrom(128)
		return float(response)

	# Get output (real) voltage of channel
	def GetOutputVoltage(self, channel):
		if channel != 1 and channel != 2:
			# Invalid channel
			print("Invalid channel")
			return
		data = "VOUT%d?\r\n" % (channel)
		self.udpSocket.sendto(str.encode(data), self.serverAddressPort)
		response, addr = self.udpSocket.recvfrom(128)
		return float(response)
	
	# Get output (real) current of channel
	def GetOutputCurrent(self, channel):
		if channel != 1 and channel != 2:
			# Invalid channel
			print("Invalid channel")
			return
		data = "IOUT%d?\r\n" % (channel)
		self.udpSocket.sendto(str.encode(data), self.serverAddressPort)
		response, addr = self.udpSocket.recvfrom(128)
		return float(response)
	
	# Enables output of channel
	def EnableOutput(self, channel):
		if channel != 1 and channel != 2:
			# Invalid channel
			print("Invalid channel")
			return
		data = "OUT%d:1\r\n" % (channel)
		self.udpSocket.sendto(str.encode(data), self.serverAddressPort)

	# Disables output of channel
	def DisableOutput(self, channel):
		if channel != 1 and channel != 2:
			# Invalid channel
			print("Invalid channel")
			return
		data = "OUT%d:0\r\n" % (channel)
		self.udpSocket.sendto(str.encode(data), self.serverAddressPort)
		

# Connect to Power Supply
powerSupply = KD3305P()

# Get Power Supply Info
powerSupply.GetDeviceInfo()

# Set voltage of channel 1
powerSupply.SetVoltage(1, 22.45)
voltage = powerSupply.GetVoltage(1)
msg = "Voltage CH1: %2.2f V" % (voltage)
print(msg)

# Set current of channel 2
powerSupply.SetCurrent(2, 0.345)
current = powerSupply.GetCurrent(2)
msg = "Current CH2: %1.3f A" % (current)
print(msg)

# Enable Output channel 1
powerSupply.EnableOutput(1)
time.sleep(5)

# Get Actual/Real output voltage and current of channel 1
voltage = powerSupply.GetOutputVoltage(1)
msg = "Output Voltage CH1: %2.2f V" % (voltage)
print(msg)

current = powerSupply.GetOutputCurrent(1)
msg = "Output Current CH1: %1.3f A" % (current)
print(msg)

